#!/bin/bash
#
# Installation: Put this script somewhere in the path, and make sure
# that the first line points to a bash binary
#

S0=`basename $0`

if [[ "x$1" == "x" || "x$1" == "x--help" || "x$1" == "x-h" ]]; then
    cat <<EOF
Syntax: $S0 [options] wiki-page-address
Example:
 $ $0 Main/HomePage		# Open Main.HomePage (using default function)

 $ $S0 -f lyx-open Main/HomePage # Use lyx-open to open page

 $ $S0 Main.HomePage -nw	# Pass extra option '-nw' to Emacs

Options:
  -v			Be more verbose
  -n			Only print commands, do not actually run
  -f function		Name of function to invoke (default: pmichaud-open)
  -e editor		Name of editor
  --version		Show version
  -h, --help		Show this help

Environment variables:

If the variable WIKI_OPEN_Function exists, it's value is taken as the
default function. E.g. setting 'WIKI_OPEN_Function=lyx-open' means you
don't have to give the options: '-l lyx-open' every time.

If the variable WIKI_OPEN_Args exists, it's value is prepended as
additional arguments send to the editor. E.g. setting
'WIKI_OPEN_Args=-nw' means you don't have to give the '-nw' option on
the command line.

EOF
exit
fi

Version='$Name: release-1_3 $';		# Assign CVS release name
[[ "$1" == --version ]] && {
    Ver_=${Version#\$Name: };
    Ver_=${Ver_% $};
    Ver_=${Ver_#release-};
    Ver_=${Ver_//_/.};
    printf "%s\n" "$Ver_"
    exit 0;
}

# Use: Verbose 1 "Only printed if Verbose >= 1, arg: %s" "an argument"
# The first numeric argument can be skipped if the condition is onl that
# the verbosity > 0
function Verbose() {
    if [[ ${#1} -eq 1 && $Verbosity -lt $1 ]]; then return; fi;
    if [[ ${#1} -ne 1 && $Verbosity -eq 0 ]]; then return; fi;
    if [[ ${#1} -eq 1 ]]; then shift; fi;
    FMT="$1\n"
    shift;
    printf "$FMT" "$@"
}

# Use: Error -1 "This argument is wrong: %s" "an argument"
function Error() {
    ERROR_CODE=$1;
    shift;
    printf "%s: Error(%d) " "`basename $0`" $ERROR_CODE
    FMT="$2\n"
    shift
    printf "$FMT" "$@"
    exit $ERROR_CODE
}

#
# Set default values of variables
#
# Standard variables
Verbosity=0			# Controls the level of verbosity
DO=1				# Is set false if nothing should be done
#
# Script specific variables
EDITOR=emacs			# Name of program to invoke
Adr=""
Function=${WIKI_OPEN_Function-pmichaud-open} # Elisp function such as lyx-open
Args=${WIKI_OPEN_Args-}		# Extra arguments for the editor

until [[ $# -eq 0 ]]; do
    case "$1" in
	-v)	let ++Verbosity ;;
	-n)	DO=0 ;;
	-f)	Function=$2; shift ;;
	-e)	EDITOR=$2; shift ;;
	-*)	Error -1 "Unknown option: %s" "$1" ;;
	*)
	    Adr=$1; shift; break;
    esac
    shift;
done
Verbose "Output is verbose (verbosity = %d)" "$Verbosity"
if (( $Verbosity >= 2 )); then
    printf "Variables in the script:\n"
    printf "DO=%s\n" "$DO"
    printf "EDITOR=%s\n" "$EDITOR"
    printf "Adr=%s\n" "$Adr"
    printf "Function=%s\n" "$Function"
    printf "\n"
fi;

# Verify validity of arguments
if [[ "x$Adr" == "x" ]]; then Error "No address given!"; fi;

# Execute proper code
if (( $DO )); then msg="Executing "; else msg="Not executing "; fi;
Verbose 1 "%s %s" "$msg" \
    "$EDITOR --execute \"($Function \"$Adr\")\" $Args $*"
if (( $DO )); then $EDITOR --execute "($Function \"$Adr\")" $Args $*; fi;
